@@ -8,14 +8,14 @@ import re
 import sys
 from urllib import request, parse
 
-__version__ = "0.2.1"
+__version__ = "0.2.2"
 
 force = False
 
 fake_headers = {
     'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
     'Accept-Charset': 'UTF-8,*;q=0.5',
-    'Accept-Encoding': 'gzip,deflate,sdc',
+    'Accept-Encoding': 'gzip,deflate,sdch',
     'Accept-Language': 'en-US,en;q=0.8',
     'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.57 Safari/537.1'
 }
@@ -43,6 +43,9 @@ def r1_of(patterns, text):
         if x:
             return x
 
+def unicodize(text):
+    return re.sub(r'\\u([0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f])', lambda x: chr(int(x.group(0)[2:], 16)), text)
+
 def escape_file_path(path):
     path = path.replace('/', '-')
     path = path.replace('\\', '-')
@@ -119,6 +122,7 @@ def url_info(url, faker = False):
         'video/3gpp': '3gp',
         'video/f4v': 'flv',
         'video/mp4': 'mp4',
+        'video/MP2T': 'ts',
         'video/webm': 'webm',
         'video/x-flv': 'flv'
     }
@@ -211,6 +215,64 @@ def url_save(url, filepath, bar, refer = None, is_part = False, faker = False):
         os.remove(filepath) # on Windows rename could fail if destination filepath exists
     os.rename(temp_filepath, filepath)
 
+def url_save_chunked(url, filepath, bar, refer = None, is_part = False, faker = False):
+    if os.path.exists(filepath):
+        if not force:
+            if not is_part:
+                if bar:
+                    bar.done()
+                print('Skipping %s: file already exists' % tr(os.path.basename(filepath)))
+            else:
+                if bar:
+                    bar.update_received(os.path.getsize(filepath))
+            return
+        else:
+            if not is_part:
+                if bar:
+                    bar.done()
+                print('Overwriting %s' % tr(os.path.basename(filepath)), '...')
+    elif not os.path.exists(os.path.dirname(filepath)):
+        os.mkdir(os.path.dirname(filepath))
+    
+    temp_filepath = filepath + '.download'
+    received = 0
+    if not force:
+        open_mode = 'ab'
+        
+        if os.path.exists(temp_filepath):
+            received += os.path.getsize(temp_filepath)
+            if bar:
+                bar.update_received(os.path.getsize(temp_filepath))
+    else:
+        open_mode = 'wb'
+    
+    if faker:
+        headers = fake_headers
+    else:
+        headers = {}
+    if received:
+        headers['Range'] = 'bytes=' + str(received) + '-'
+    if refer:
+        headers['Referer'] = refer
+    
+    response = request.urlopen(request.Request(url, headers = headers), None)
+    
+    with open(temp_filepath, open_mode) as output:
+        while True:
+            buffer = response.read(1024 * 256)
+            if not buffer:
+                break
+            output.write(buffer)
+            received += len(buffer)
+            if bar:
+                bar.update_received(len(buffer))
+    
+    assert received == os.path.getsize(temp_filepath), '%s == %s == %s' % (received, os.path.getsize(temp_filepath))
+    
+    if os.access(filepath, os.W_OK):
+        os.remove(filepath) # on Windows rename could fail if destination filepath exists
+    os.rename(temp_filepath, filepath)
+
 class SimpleProgressBar:
     def __init__(self, total_size, total_pieces = 1):
         self.displayed = False
@@ -315,12 +377,12 @@ def download_urls(urls, title, ext, total_size, output_dir = '.', refer = None,
         url_save(url, filepath, bar, refer = refer, faker = faker)
         bar.done()
     else:
-        flvs = []
+        parts = []
         print('Downloading %s.%s ...' % (tr(title), ext))
         for i, url in enumerate(urls):
             filename = '%s[%02d].%s' % (title, i, ext)
             filepath = os.path.join(output_dir, filename)
-            flvs.append(filepath)
+            parts.append(filepath)
             #print 'Downloading %s [%s/%s]...' % (tr(filename), i + 1, len(urls))
             bar.update_piece(i + 1)
             url_save(url, filepath, bar, refer = refer, is_part = True, faker = faker)
@@ -329,15 +391,59 @@ def download_urls(urls, title, ext, total_size, output_dir = '.', refer = None,
             print()
             return
         if ext == 'flv':
-            from .processor.merge_flv import concat_flvs
-            concat_flvs(flvs, os.path.join(output_dir, title + '.flv'))
-            for flv in flvs:
-                os.remove(flv)
+            from .processor.join_flv import concat_flv
+            concat_flv(parts, os.path.join(output_dir, title + '.flv'))
+            for part in parts:
+                os.remove(part)
         elif ext == 'mp4':
-            from .processor.merge_mp4 import concat_mp4s
-            concat_mp4s(flvs, os.path.join(output_dir, title + '.mp4'))
-            for flv in flvs:
-                os.remove(flv)
+            from .processor.join_mp4 import concat_mp4
+            concat_mp4(parts, os.path.join(output_dir, title + '.mp4'))
+            for part in parts:
+                os.remove(part)
+        else:
+            print("Can't merge %s files" % ext)
+    
+    print()
+
+def download_urls_chunked(urls, title, ext, total_size, output_dir = '.', refer = None, merge = True, faker = False):
+    assert urls
+    assert ext in ('ts')
+    title = escape_file_path(title)
+    filename = '%s.%s' % (title, ext)
+    filepath = os.path.join(output_dir, filename)
+    if total_size:
+        if not force and os.path.exists(filepath) and os.path.getsize(filepath) >= total_size * 0.9:
+            print('Skipping %s: file already exists' % tr(filepath))
+            print()
+            return
+        bar = SimpleProgressBar(total_size, len(urls))
+    else:
+        bar = PiecesProgressBar(total_size, len(urls))
+    
+    if len(urls) == 1:
+        url = urls[0]
+        print('Downloading %s ...' % tr(filename))
+        url_save_chunked(url, filepath, bar, refer = refer, faker = faker)
+        bar.done()
+    else:
+        parts = []
+        print('Downloading %s.%s ...' % (tr(title), ext))
+        for i, url in enumerate(urls):
+            filename = '%s[%02d].%s' % (title, i, ext)
+            filepath = os.path.join(output_dir, filename)
+            parts.append(filepath)
+            #print 'Downloading %s [%s/%s]...' % (tr(filename), i + 1, len(urls))
+            bar.update_piece(i + 1)
+            url_save_chunked(url, filepath, bar, refer = refer, is_part = True, faker = faker)
+        bar.done()
+        if not merge:
+            print()
+            return
+        if ext == 'ts':
+            from .processor.join_ts import concat_ts
+            concat_ts(parts, os.path.join(output_dir, title + '.ts'))
+            for part in parts:
+                os.remove(part)
         else:
             print("Can't merge %s files" % ext)
     
@@ -355,6 +461,8 @@ def print_info(site_info, title, type, size):
         type = 'video/x-flv'
     elif type in ['mp4']:
         type = 'video/mp4'
+    elif type in ['ts']:
+        type = 'video/MP2T'
     elif type in ['webm']:
         type = 'video/webm'
     
@@ -364,6 +472,8 @@ def print_info(site_info, title, type, size):
         type_info = "Flash video (%s)" % type
     elif type in ['video/mp4', 'video/x-m4v']:
         type_info = "MPEG-4 video (%s)" % type
+    elif type in ['video/MP2T']:
+        type_info = "MPEG-2 transport stream (%s)" % type
     elif type in ['video/webm']:
         type_info = "WebM video (%s)" % type
     #elif type in ['video/ogg']:
@@ -465,6 +575,8 @@ def script_main(script_name, download, download_playlist = None):
     set_http_proxy(proxy)
     
     for url in args:
+        if url.startswith('https://'):
+            url = url[8:]
         if not url.startswith('http://'):
             url = 'http://' + url
         
